@@ -25,6 +25,7 @@
 #endif
 
 #include "gtkhtml.h"
+#include "gtkhtml-private.h"
 
 #include "htmlclueflow.h"
 #include "htmlcursor.h"
@@ -50,6 +51,7 @@
 #include "table.h"
 #include "text.h"
 
+/* #define DEBUG */
 #ifdef DEBUG
 #include "gtkhtmldebug.h"
 #endif
@@ -301,7 +303,7 @@ spell_add (GtkWidget *mi, GtkHTMLControlData *cd)
 
 	word = html_engine_get_spell_word (e);
 	if (word) {
-		spell_add_to_personal (cd->html, word, cd);
+		spell_add_to_personal (cd->html, word, g_object_get_data (G_OBJECT (mi), "abbrev"), cd);
 		g_free (word);
 	}
 	html_engine_spell_check (e);
@@ -348,15 +350,19 @@ insert_html (GtkWidget *mi, GtkHTMLControlData *cd)
 		g_signal_connect (menuitem, "activate", G_CALLBACK (f), cd); \
                 (*items)++; items_sep++
 
-#define ADD_ITEM_SENSITIVE(l,f,t,s) \
-		menuitem = gtk_menu_item_new_with_label (l); \
-                ADD_ITEM_BASE (f,t); \
-                gtk_widget_set_sensitive (menuitem, s);
-
 #define ADD_ITEM(l,f,t) \
 		menuitem = gtk_menu_item_new_with_label (l); \
                 ADD_ITEM_BASE (f,t)
 
+#define ADD_STOCK(i,f) \
+		menuitem = gtk_image_menu_item_new_from_stock (i, NULL); \
+                ADD_ITEM_BASE (f,NONE);
+
+#define ADD_STOCK_SENSITIVE(i,f,s) \
+		menuitem = gtk_image_menu_item_new_from_stock (i, NULL); \
+                ADD_ITEM_BASE (f,NONE); \
+                gtk_widget_set_sensitive (menuitem, s);
+
 #define ADD_SEP \
         if (items_sep) { \
                 menuitem = gtk_menu_item_new (); \
@@ -383,6 +389,34 @@ insert_html (GtkWidget *mi, GtkHTMLControlData *cd)
 			gtk_widget_show (menu); \
 			menu = menuparent
 
+static gint
+get_n_languages (GtkHTMLControlData *cd)
+{
+	gint i, n = 0;
+
+	if (cd->languages)
+		for (i = 0; i < cd->languages->_length; i ++)
+			if (strstr (html_engine_get_language (cd->html->engine), cd->languages->_buffer [i].abbreviation))
+				n ++;
+
+	return n;
+}
+
+static const gchar *
+get_language (GtkHTMLControlData *cd)
+{
+	gint i;
+	const gchar *abbrev = NULL;
+
+	if (cd->languages)
+		for (i = 0; i < cd->languages->_length; i ++)
+			if (strstr (html_engine_get_language (cd->html->engine), cd->languages->_buffer [i].abbreviation)) {
+				abbrev = cd->languages->_buffer [i].abbreviation;
+			}
+
+	return abbrev;
+}
+
 static GtkWidget *
 prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items, guint *props)
 {
@@ -411,13 +445,13 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items, guint *props)
 	ADD_SEP;
 #endif
 	active = html_engine_is_selection_active (e);
-	ADD_ITEM (_("Undo"), undo, NONE); 
-	ADD_ITEM (_("Redo"), redo, NONE); 
+	ADD_STOCK (GTK_STOCK_UNDO, undo); 
+	ADD_STOCK (GTK_STOCK_REDO, redo); 
 
 	ADD_SEP;
-	ADD_ITEM_SENSITIVE (_("Copy"), copy, NONE, active);
-	ADD_ITEM_SENSITIVE (_("Cut"),  cut, NONE, active);
-	ADD_ITEM (_("Paste"),  paste, NONE);
+	ADD_STOCK_SENSITIVE (GTK_STOCK_CUT,  cut, active);
+	ADD_STOCK_SENSITIVE (GTK_STOCK_COPY, copy, active);
+	ADD_STOCK (GTK_STOCK_PASTE, paste);
 	ADD_ITEM (_("Paste Quotation"),  paste_cite, NONE);
 
 	ADD_SEP;
@@ -432,57 +466,30 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items, guint *props)
 		ADD_ITEM (_("Remove link"), remove_link, NONE);
 	}
 
-	if (!active && obj && html_object_is_text (obj)
-	    && !html_engine_spell_word_is_valid (e)) {
-		gchar *spell, *word, *ignore, *add;
-
-		word   = html_engine_get_spell_word (e);
-		spell  = g_strdup_printf (_("Check '%s' spelling..."), word);
-		add    = g_strdup_printf (_("Add '%s' to dictionary"), word);
-		ignore = g_strdup_printf (_("Ignore '%s'"), word);
-		ADD_SEP;
-		SUBMENU (N_("Spell checker"));
-		if (cd->has_spell_control) {
-			ADD_ITEM (spell, spell_check_cb, NONE);
-		} else {
-			ADD_ITEM (_("Suggest word"), spell_suggest, NONE);
-		}
-		ADD_ITEM (add, spell_add, NONE);
-		ADD_ITEM (ignore, spell_ignore, NONE);
-		END_SUBMENU;
-
-		g_free (spell);
-		g_free (add);
-		g_free (ignore);
-		g_free (word);
-	}
-
 	if (cd->format_html && obj) {
+		ADD_SEP;
+		SUBMENU ("Style");
 		switch (HTML_OBJECT_TYPE (obj)) {
 		case HTML_TYPE_TEXT:
-			ADD_SEP;
-			ADD_ITEM (_("Text..."), prop_dialog, TEXT);
+			ADD_ITEM (_("Text Style..."), prop_dialog, TEXT);
 			ADD_PROP (TEXT);
-			ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
+			ADD_ITEM (_("Paragraph Style..."), prop_dialog, PARAGRAPH);
 			ADD_PROP (PARAGRAPH);
 			break;
 		case HTML_TYPE_LINKTEXT:
-			ADD_SEP;
-			ADD_ITEM (_("Link..."), link_prop_dialog, LINK);
+			ADD_ITEM (_("Link Style..."), link_prop_dialog, LINK);
 			ADD_PROP (LINK);
-			ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
+			ADD_ITEM (_("Paragraph Style..."), prop_dialog, PARAGRAPH);
 			ADD_PROP (PARAGRAPH);
 			break;
 		case HTML_TYPE_RULE:
-			ADD_SEP;
-			ADD_ITEM (_("Rule..."), prop_dialog, RULE);
+			ADD_ITEM (_("Rule Style..."), prop_dialog, RULE);
 			ADD_PROP (RULE);
 			break;
 		case HTML_TYPE_IMAGE:
-			ADD_SEP;
-			ADD_ITEM (_("Image..."), prop_dialog, IMAGE);
+			ADD_ITEM (_("Image Style..."), prop_dialog, IMAGE);
 			ADD_PROP (IMAGE);
-			ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
+			ADD_ITEM (_("Paragraph Style..."), prop_dialog, PARAGRAPH);
 			ADD_PROP (PARAGRAPH);
 			break;
 		default:
@@ -490,14 +497,21 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items, guint *props)
 		}
 		if (obj->parent && obj->parent->parent && HTML_IS_TABLE_CELL (obj->parent->parent)) {
 			if (cd->format_html) {
-				ADD_SEP;
 				ADD_PROP (CELL);
-				ADD_ITEM (_("Cell..."), prop_dialog, CELL);
+				ADD_ITEM (_("Cell Style..."), prop_dialog, CELL);
 				if (obj->parent->parent->parent && HTML_IS_TABLE (obj->parent->parent->parent)) {
 					ADD_PROP (TABLE);
-					ADD_ITEM (_("Table..."), prop_dialog, TABLE);
+					ADD_ITEM (_("Table Style..."), prop_dialog, TABLE);
 				}
 			}
+		}
+		if (cd->format_html) {
+			ADD_PROP (BODY);
+			ADD_ITEM (_("Page Style..."), prop_dialog, BODY);
+		}
+		END_SUBMENU;
+		ADD_SEP;
+		if (obj->parent && obj->parent->parent && HTML_IS_TABLE_CELL (obj->parent->parent)) {
 			SUBMENU (N_("Table insert"));
 			ADD_ITEM (_("Table"), insert_table_cb, NONE);
 			ADD_SEP;
@@ -516,12 +530,40 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items, guint *props)
 		}
 	}
 
-	if (cd->format_html) {
+	if (!active && obj && html_object_is_text (obj)
+	    && !html_engine_spell_word_is_valid (e)) {
+		gchar *spell, *word, *ignore, *add;
+
 		ADD_SEP;
-		ADD_PROP (BODY);
-		ADD_ITEM (_("Page..."), prop_dialog, BODY);
+		ADD_ITEM ("Check Word Spelling...", spell_check_cb, NONE);
+		if (get_n_languages (cd) > 1) {
+			gchar *lang;
+			gint i;
+
+			SUBMENU ("Add Word to");
+
+			for (i = 0; i < cd->languages->_length; i ++) {
+				if (strstr (html_engine_get_language (cd->html->engine), cd->languages->_buffer [i].abbreviation)) {
+					lang = g_strdup_printf (_("%s Dictionary"), cd->languages->_buffer [i].name);
+					ADD_ITEM (lang, spell_add, NONE);
+					g_object_set_data (G_OBJECT (menuitem), "abbrev", cd->languages->_buffer [i].abbreviation);
+					g_free (lang);
+				}
+			}
+			
+			END_SUBMENU;
+		} else {
+			ADD_ITEM ("Add Word to Dictionary", spell_add, NONE);
+			g_object_set_data (G_OBJECT (menuitem), "abbrev", (gpointer) get_language (cd));
+		}
+		ADD_ITEM ("Ignore Misspelled Word", spell_ignore, NONE);
 	}
 
+	SUBMENU ("Input Methods");
+	gtk_im_multicontext_append_menuitems (GTK_IM_MULTICONTEXT (cd->html->priv->im_context), 
+					      GTK_MENU_SHELL (menu));
+	END_SUBMENU;
+
 	if (*items == 0) {
 		gtk_object_sink (GTK_OBJECT (menu));
 		menu = NULL;
