@@ -23,6 +23,7 @@
 
 #include <config.h>
 #include "dialog.h"
+#include <libgnomeui/gnome-window-icon.h>
 #include "properties.h"
 
 #define GTK_HTML_EDIT_IMAGE_BWIDTH      0
@@ -36,16 +37,6 @@
 #define ALIGN_CENTER 1
 #define ALIGN_BOTTOM 2
 
-struct _GtkHTMLEditPropertiesDialog {
-	GtkWidget           *dialog;
-	GtkHTMLControlData  *control_data;
-
-	GList               *page_data;
-	GtkWidget           *notebook;
-	gboolean             insert;
-	gchar               *title;
-};
-
 struct _PageData {
 	GtkHTMLEditPropertyType      type;
 	GtkHTMLEditPropertyApplyFunc apply;
@@ -58,32 +49,29 @@ typedef struct _PageData PageData;
 static void
 apply_cb (PageData *pd, GtkHTMLEditPropertiesDialog *d)
 {
-	(*pd->apply) (d->control_data, pd->data);
+	if (!(*pd->apply) (d->control_data, pd->data))
+		d->all_changes_applied = FALSE;
 }
 
 static void
-apply (GtkWidget *w, GtkHTMLEditPropertiesDialog *d)
+apply (GtkHTMLEditPropertiesDialog *d)
 {
+	d->all_changes_applied = TRUE;
 	g_list_foreach (d->page_data, (GFunc) apply_cb, d);
-	gnome_dialog_set_sensitive (GNOME_DIALOG (d->dialog), 0, FALSE);
-	if (!d->insert)
-		gnome_dialog_set_sensitive (GNOME_DIALOG (d->dialog), 1, FALSE);
+	if (d->all_changes_applied) {
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (d->dialog), 0, FALSE);
+		if (!d->insert)
+			gtk_dialog_set_response_sensitive (GTK_DIALOG (d->dialog), 1, FALSE);
+	}
 }
 
 static void
-prop_close (GtkWidget *w, GtkHTMLEditPropertiesDialog *d)
+prop_close (GtkHTMLEditPropertiesDialog *d)
 {
-	gnome_dialog_close (GNOME_DIALOG (d->dialog));
+	gtk_dialog_response (GTK_DIALOG (d->dialog), GTK_RESPONSE_CLOSE);
 	gtk_html_edit_properties_dialog_destroy (d);
 }
 
-static void
-ok (GtkWidget *w, GtkHTMLEditPropertiesDialog *d)
-{
-	apply (w, d);
-	prop_close (w,d);
-}
-
 static void
 switch_page (GtkWidget *w, GtkNotebookPage *page, gint num, GtkHTMLEditPropertiesDialog *d)
 {
@@ -103,9 +91,28 @@ switch_page (GtkWidget *w, GtkNotebookPage *page, gint num, GtkHTMLEditPropertie
 static void
 destroy_dialog (GtkWidget *w, gpointer data)
 {
+	printf ("destroy\n");
 	((GtkHTMLEditPropertiesDialog *) data)->dialog = NULL;
 }
 
+static void
+dialog_response (GtkDialog *dialog, gint response_id, GtkHTMLEditPropertiesDialog *d)
+{
+	switch (response_id) {
+	case GTK_RESPONSE_CLOSE:
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		break;
+	case 0: /* OK */
+		apply (d);
+		prop_close (d);
+		break;
+	case 1: /* Insert/Apply */
+		apply (d);
+		if (d->insert)
+			prop_close (d);
+	}
+}
+
 GtkHTMLEditPropertiesDialog *
 gtk_html_edit_properties_dialog_new (GtkHTMLControlData *cd, gboolean insert, gchar *title, gchar *icon_path)
 {
@@ -116,34 +123,28 @@ gtk_html_edit_properties_dialog_new (GtkHTMLControlData *cd, gboolean insert, gc
 	d->title          = g_strdup (title);
 	d->insert         = insert;
 	d->control_data   = cd;
-	d->dialog         = (insert) ? gnome_dialog_new (title,
-							 _("Insert"),
-							 GNOME_STOCK_BUTTON_CLOSE, NULL)
-		:  gnome_dialog_new (title,
-				     GNOME_STOCK_BUTTON_OK,
-				     GNOME_STOCK_BUTTON_APPLY,
-				     GNOME_STOCK_BUTTON_CLOSE, NULL);
 	parent = get_parent_window (GTK_WIDGET (cd->html));
-	if (parent) {
-		gnome_dialog_set_parent (GNOME_DIALOG (d->dialog), parent);
-	}
+	d->dialog         = insert ? gtk_dialog_new_with_buttons (title, parent, 0,
+								  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+								  _("Insert"), 1,
+								  NULL)
+		:  gtk_dialog_new_with_buttons (title, parent, 0,
+						GTK_STOCK_APPLY, 1,
+						GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+						GTK_STOCK_OK, 0,
+						NULL);
 	d->notebook = gtk_notebook_new ();
-	gtk_signal_connect (GTK_OBJECT (d->dialog), "destroy", destroy_dialog, d);
-	gtk_signal_connect (GTK_OBJECT (d->notebook), "switch_page", switch_page, d);
-	gtk_box_pack_start_defaults (GTK_BOX (GNOME_DIALOG (d->dialog)->vbox), d->notebook);
+	g_signal_connect (d->dialog, "destroy", G_CALLBACK (destroy_dialog), d);
+	g_signal_connect (d->notebook, "switch_page", G_CALLBACK (switch_page), d);
+	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), d->notebook);
 	gtk_widget_show (d->notebook);
 
-	gnome_dialog_button_connect (GNOME_DIALOG (d->dialog), 0, ok, d);
-	if (!insert)
-		gnome_dialog_button_connect (GNOME_DIALOG (d->dialog), 1, apply, d);
-	gnome_dialog_button_connect (GNOME_DIALOG (d->dialog), insert ? 1 : 2, prop_close, d);
-	gnome_dialog_set_default (GNOME_DIALOG (d->dialog), 0);
+	g_signal_connect (d->dialog, "response", G_CALLBACK (dialog_response), d);
 
-	
 	gnome_window_icon_set_from_file (GTK_WINDOW (d->dialog), icon_path);
-	gnome_dialog_set_sensitive (GNOME_DIALOG (d->dialog), 0, FALSE);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (d->dialog), 0, FALSE);
 	if (!insert)
-		gnome_dialog_set_sensitive (GNOME_DIALOG (d->dialog), 1, FALSE);
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (d->dialog), 1, FALSE);
 
 	return d;
 }
@@ -195,7 +196,6 @@ gtk_html_edit_properties_dialog_add_entry (GtkHTMLEditPropertiesDialog *d,
 void
 gtk_html_edit_properties_dialog_show (GtkHTMLEditPropertiesDialog *d)
 {
-	gtk_window_set_modal (GTK_WINDOW (d->dialog), TRUE);
 	if (g_list_length (d->page_data) <= 1) {
 		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (d->notebook), FALSE);
 		gtk_notebook_set_show_border (GTK_NOTEBOOK (d->notebook), FALSE);
@@ -206,17 +206,16 @@ gtk_html_edit_properties_dialog_show (GtkHTMLEditPropertiesDialog *d)
 void
 gtk_html_edit_properties_dialog_close (GtkHTMLEditPropertiesDialog *d)
 {
-	/* gtk_window_set_modal (GTK_WINDOW (d->dialog), FALSE); */
 	if (d->dialog)
-		gnome_dialog_close (GNOME_DIALOG (d->dialog));
+		gtk_dialog_response (GTK_DIALOG (d->dialog), GTK_RESPONSE_CLOSE);
 }
 
 void
 gtk_html_edit_properties_dialog_change (GtkHTMLEditPropertiesDialog *d)
 {
-	gnome_dialog_set_sensitive (GNOME_DIALOG (d->dialog), 0, TRUE);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (d->dialog), 0, TRUE);
 	if (!d->insert)
-		gnome_dialog_set_sensitive (GNOME_DIALOG (d->dialog), 1, TRUE);
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (d->dialog), 1, TRUE);
 }
 
 static gint
@@ -231,5 +230,5 @@ gtk_html_edit_properties_dialog_set_page (GtkHTMLEditPropertiesDialog *d, GtkHTM
 	gint pos = g_list_position (d->page_data, g_list_find_custom (d->page_data, GINT_TO_POINTER (t),
 								      (GCompareFunc) find_type));
 	if (pos >= 0)
-		gtk_notebook_set_page (GTK_NOTEBOOK (d->notebook), pos);
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (d->notebook), pos);
 }
