@@ -24,11 +24,11 @@
 
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-job.h>
+#include <gtk/gtkimcontext.h>
 #include <gtk/gtkwidget.h>
 #include "gtkhtml-types.h"
 #include "htmltypes.h"
 
-
 struct _GtkHTMLPrivate {
 	guint idle_handler_id;
 	guint scroll_timeout_id;
@@ -53,10 +53,8 @@ struct _GtkHTMLPrivate {
 	guint notify_id;
 	guint notify_spell_id;
 
-#ifdef GTK_HTML_USE_XIM
-	GdkICAttr *ic_attr;
-	GdkIC *ic;
-#endif
+	GtkIMContext *im_context;
+	gboolean need_im_reset;
 
 	HTMLObject *dnd_object;
 	HTMLObject *dnd_real_object;
@@ -77,5 +75,6 @@ void  gtk_html_editor_event             (GtkHTML                 *html,
 					 GtkHTMLEditorEventType   event,
 					 GValue                  *args);
 void  gtk_html_api_set_language         (GtkHTML                 *html);
+void  gtk_html_im_reset                 (GtkHTML                 *html);
 
 #endif /* _GTKHTML_PRIVATE_H */
