@@ -349,6 +349,8 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 		
 		html_iframe_set_scrolling (iframe, GTK_POLICY_NEVER);
 
+		e->width = width;
+		e->height = height;
 		o->width = width;
 		o->ascent = height;
 		o->descent = 0;
@@ -422,18 +424,10 @@ check_point (HTMLObject *self,
 	    || y >= self->y + self->descent || y < self->y - self->ascent)
 		return NULL;
 
-	x -= self->x + e->leftBorder - e->x_offset;
-	y -= self->y - self->ascent + e->topBorder - e->y_offset;
+	x -= self->x - e->x_offset;
+	y -= self->y - self->ascent - e->y_offset;
 
-	if (for_cursor && (x < 0 || y < e->clue->y - e->clue->ascent))
-		return html_object_check_point (e->clue, e->painter, 0, e->clue->y - e->clue->ascent,
-						offset_return, for_cursor);
-
-	if (for_cursor && (x > e->clue->width - 1 || y > e->clue->y + e->clue->descent - 1))
-		return html_object_check_point (e->clue, e->painter, e->clue->width - 1, e->clue->y + e->clue->descent - 1,
-						offset_return, for_cursor);
-
-	return html_object_check_point (e->clue, e->painter, x, y, offset_return, for_cursor);
+	return html_engine_get_object_at (e, x, y, offset_return, for_cursor);
 }
 
 static gboolean
