@@ -22,6 +22,7 @@
 #include <config.h>
 #include "gtkhtml-compat.h"
 
+#include <string.h>
 #include <gnome.h>
 #include <ctype.h>
 #include <libgnomeprint/gnome-print.h>
@@ -478,6 +479,27 @@ draw_background (HTMLPainter *painter,
 	}
 }
 
+static void
+print_pixbuf (GnomePrintContext *pc, GdkPixbuf *pixbuf)
+{
+	if (!pixbuf || (gdk_pixbuf_get_colorspace (pixbuf) != GDK_COLORSPACE_RGB))
+		return;
+	
+	if (gdk_pixbuf_get_has_alpha (pixbuf)) {
+		gnome_print_rgbaimage (pc, 
+				       gdk_pixbuf_get_pixels (pixbuf),
+				       gdk_pixbuf_get_width (pixbuf),
+				       gdk_pixbuf_get_height (pixbuf),
+				       gdk_pixbuf_get_rowstride (pixbuf));
+	} else {
+		gnome_print_rgbimage (pc, 
+				      gdk_pixbuf_get_pixels (pixbuf),
+				      gdk_pixbuf_get_width (pixbuf),
+				      gdk_pixbuf_get_height (pixbuf),
+				      gdk_pixbuf_get_rowstride (pixbuf));
+	}
+}
+
 static void
 draw_pixmap (HTMLPainter *painter, GdkPixbuf *pixbuf, gint x, gint y, gint scale_width, gint scale_height, const GdkColor *color)
 {
@@ -500,7 +522,7 @@ draw_pixmap (HTMLPainter *painter, GdkPixbuf *pixbuf, gint x, gint y, gint scale
 	gnome_print_gsave (printer->context);
 	gnome_print_translate (printer->context, print_x, print_y - print_scale_height);
 	gnome_print_scale (printer->context, print_scale_width, print_scale_height);
-	/* FIX2 gnome_print_pixbuf (printer->context, pixbuf); */
+	print_pixbuf (printer->context, pixbuf);
 	gnome_print_grestore (printer->context);
 }
 
@@ -530,28 +552,33 @@ fill_rect (HTMLPainter *painter, gint x, gint y, gint width, gint height)
 	gnome_print_fill (printer->context);
 }
 
-static void
-draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GList *items, GList *glyphs)
+static gint
+draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GList *items, GList *glyphs, gint start_byte_offset)
 {
 	GnomeFont *font;
 	HTMLPrinter *printer;
+	gint bytes;
 	gdouble print_x, print_y;
+	double text_width, asc, dsc;
 
 	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->context != NULL);
+	g_return_val_if_fail (printer->context != NULL, 0);
 
+
+	bytes = g_utf8_offset_to_pointer (text, len) - text;
+	font = html_painter_get_font (painter, painter->font_face, painter->font_style);
+	dsc = gnome_font_get_descender (font);
+	asc = gnome_font_get_ascender (font);
 	html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
+	print_y -= dsc;
 
 	gnome_print_newpath (printer->context);
 	gnome_print_moveto (printer->context, print_x, print_y);
-
-	font = html_painter_get_font (painter, painter->font_face, painter->font_style);
 	gnome_print_setfont (printer->context, font);
-	gnome_print_show_sized (printer->context, text, g_utf8_offset_to_pointer (text, len) - text);
+	gnome_print_show_sized (printer->context, text, bytes);
 
+	text_width = gnome_font_get_width_utf8_sized (font, text, bytes);
 	if (painter->font_style & (GTK_HTML_FONT_STYLE_UNDERLINE | GTK_HTML_FONT_STYLE_STRIKEOUT)) {
-		double text_width;
-		double ascender, descender;
 		double y;
 
 		gnome_print_gsave (printer->context);
@@ -560,9 +587,7 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GL
 		gnome_print_setlinewidth (printer->context, 1.0);
 		gnome_print_setlinecap (printer->context, GDK_CAP_BUTT);
 
-		text_width = gnome_font_get_width_utf8_sized (font, text, len);
 		if (painter->font_style & GTK_HTML_FONT_STYLE_UNDERLINE) {
-			descender = gnome_font_get_descender (font);
 			y = print_y + gnome_font_get_underline_position (font);
 
 			gnome_print_newpath (printer->context);
@@ -574,8 +599,7 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GL
 		}
 
 		if (painter->font_style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
-			ascender = gnome_font_get_ascender (font);
-			y = print_y + ascender / 2.0;
+			y = print_y + asc / 2.0;
 			gnome_print_newpath (printer->context);
 			gnome_print_moveto (printer->context, print_x, y);
 			gnome_print_lineto (printer->context, print_x + text_width, y);
@@ -586,6 +610,8 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GL
 
 		gnome_print_grestore (printer->context);
 	}
+
+	return SCALE_GNOME_PRINT_TO_ENGINE (text_width);
 }
 
 static void
@@ -625,7 +651,7 @@ draw_shade_line (HTMLPainter *painter,
 }
 
 static void
-calc_text_size (HTMLPainter *painter, const gchar *text, guint len, GList *items, GList *glyphs,
+calc_text_size (HTMLPainter *painter, const gchar *text, guint len, GList *items, GList *glyphs, gint start_byte_offset,
 		GtkHTMLFontStyle style, HTMLFontFace *face, gint *width, gint *asc, gint *dsc)
 {
 	HTMLPrinter *printer;
@@ -644,7 +670,7 @@ calc_text_size (HTMLPainter *painter, const gchar *text, guint len, GList *items
 }
 
 static void
-calc_text_size_bytes (HTMLPainter *painter, const gchar *text, guint len, GList *items, GList *glyphs,
+calc_text_size_bytes (HTMLPainter *painter, const gchar *text, guint len, GList *items, GList *glyphs, gint start_byte_offset,
 		      HTMLFont *font, GtkHTMLFontStyle style, gint *width, gint *asc, gint *dsc)
 {
 	HTMLPrinter *printer;
@@ -696,11 +722,13 @@ alloc_font (HTMLPainter *painter, gchar *face, gdouble size, gboolean points, Gt
 	}
 
 	return font ? html_font_new (font,
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, " ", 1)),
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, "\xc2\xa0", 2)),
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, "\t", 1)),
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, HTML_BLOCK_CITE, strlen (HTML_BLOCK_CITE))),
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, HTML_BLOCK_INDENT, strlen (HTML_BLOCK_INDENT))))
+				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, " ", 1)/HTML_PRINTER (printer)->scale),
+				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, "\xc2\xa0", 2)/HTML_PRINTER (printer)->scale),
+				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, "\t", 1)/HTML_PRINTER (printer)->scale),
+				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, HTML_BLOCK_CITE, strlen (HTML_BLOCK_CITE))
+								       /HTML_PRINTER (printer)->scale),
+				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, HTML_BLOCK_INDENT, strlen (HTML_BLOCK_INDENT))
+								       /HTML_PRINTER (printer)->scale))
 		: NULL;
 }
 
