@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+ /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* This file is part of the GtkHTML library.
 
    Copyright (C) 1997 Martin Jones (mjones@kde.org)
@@ -345,6 +345,7 @@ html_text_op_cut_helper (HTMLText *text, HTMLEngine *e, GList *from, GList *to,
 		text->spell_errors = remove_spell_errors (text->spell_errors, begin, end - begin);
 		move_spell_errors (text->spell_errors, end, - (end - begin));
 		html_text_convert_nbsp (text, TRUE);
+		items_destroy (text);
 	} else {
 		text->spell_errors = remove_spell_errors (text->spell_errors, 0, text->text_len);
 		html_object_move_cursor_before_remove (HTML_OBJECT (text), e);
@@ -513,7 +514,6 @@ const gchar *
 html_utf8_strnchr (const gchar *s, gchar c, gint len, gint *offset)
 {
 	const gchar *res = NULL;
-	gchar *next;
 
 	*offset = 0;
 	while (s && *s && *offset < len) {
@@ -529,35 +529,47 @@ html_utf8_strnchr (const gchar *s, gchar c, gint len, gint *offset)
 }
 
 gint
-html_text_text_line_length (const gchar *text, gint *line_offset, guint len)
+html_text_text_line_length (const gchar *text, gint *line_offset, guint len, gint *tabs)
 {
 	const gchar *tab, *found_tab;
 	gint cl, l, skip, sum_skip;
 
-	/* printf ("lo: %d o: %d t: '%s'\n", line_offset, len, text); */
+	/* printf ("lo: %d len: %d t: '%s'\n", line_offset, len, text); */
+	if (tabs)
+		*tabs = 0;
 	l = 0;
-	sum_skip = 0;
+	sum_skip = skip = 0;
 	tab = text;
 	while (tab && (found_tab = html_utf8_strnchr (tab, '\t', len - l, &cl)) && l < len) {
 		l   += cl;
 		if (l >= len)
 			break;
-		*line_offset  += cl;
-		skip = 8 - (*line_offset % 8);
+		if (*line_offset != -1) {
+			*line_offset  += cl;
+			skip = 8 - (*line_offset % 8);
+		}
 		tab  = found_tab + 1;
+
 		*line_offset  += skip;
-		sum_skip += skip - 1;
+		if (*line_offset != -1)
+			sum_skip += skip - 1;
 		l ++;
+		if (tabs)
+			(*tabs) ++;
 	}
 
+	if (*line_offset != -1)
+		(*line_offset) += len - l;
+	/* printf ("ll: %d\n", len + sum_skip); */
+
 	return len + sum_skip;
 }
 
 static guint
 get_line_length (HTMLObject *self, HTMLPainter *p, gint line_offset)
 {
-	return html_clueflow_tabs (HTML_CLUEFLOW (self->parent), p) || line_offset == -1
-		? html_text_text_line_length (HTML_TEXT (self)->text, &line_offset, HTML_TEXT (self)->text_len)
+	return html_clueflow_tabs (HTML_CLUEFLOW (self->parent), p)
+		? html_text_text_line_length (HTML_TEXT (self)->text, &line_offset, HTML_TEXT (self)->text_len, NULL)
 		: HTML_TEXT (self)->text_len;
 }
 
@@ -588,7 +600,7 @@ word_size (gint cl, gint so, gint eo, GList **items, GList **glyphs, gint *width
 {
 	PangoItem *item;
 	PangoRectangle rect;
-	gint ceo, offset;
+	gint ceo;
 
 	*width = *asc = *dsc = 0;
 	while (so < eo) {
@@ -642,33 +654,38 @@ calc_word_width (HTMLText *text, HTMLPainter *painter, gint line_offset)
 	}
 
 	/* printf ("calc ww m1\n"); */
-	begin = text->text;
+	begin = end = text->text;
 	start_offset = end_offset = 0;
 	il = items;
 	gl = glyphs;
 	cl = 0;
 	for (i = 0; i < text->words; i++) {
-		end   = strchr (begin + (i ? 1 : 0), ' ');
+		while (*end && *end != ' ') {
+			end = g_utf8_next_char (end);
+			end_offset++;
+		}			
 
 		if (il && gl) {
-			PangoRectangle log_rect;
-
-			/* end_offset = start_offset + (end ? g_utf8_pointer_to_offset (begin, end) : g_utf8_strlen (begin, -1)); */
-			end_offset = start_offset + (end ? g_utf8_pointer_to_offset (begin, end) : g_utf8_strlen (begin, -1));
-			/* printf ("start offset: %d (%d)\n", start_offset, end_offset - start_offset); */
 			cl = word_size (cl, start_offset, end_offset, &il, &gl, &width, &asc, &dsc);
 		} else
 			html_painter_calc_text_size_bytes (painter,
-							   begin, end ? end - begin : strlen (begin), NULL, NULL,
-							   &line_offset, font, style, &width, &asc, &dsc);
+							   begin, end - begin, NULL, NULL, 0,
+							   NULL, font, style, &width, &asc, &dsc);
+
 		text->word_width [i] = (i ? text->word_width [i - 1] : 0) + width;
 
 		if (obj->ascent < asc)
 			obj->ascent = asc;
 		if (obj->descent < dsc)
 			obj->descent = dsc;
+
 		begin = end;
 		start_offset = end_offset;
+
+		if (*end) {
+			end = g_utf8_next_char (end);
+			end_offset++;
+		}
 	}
 	/* printf ("calc ww m2\n"); */
 
@@ -677,9 +694,8 @@ calc_word_width (HTMLText *text, HTMLPainter *painter, gint line_offset)
 		glyphs = NULL;
 	}
 	if (text->text_len == 0) {
-		gint lo = 0;
 		/* FIXME: cache items and glyphs? */
-		html_painter_calc_text_size_bytes (painter, " ", 1, NULL, NULL, &lo, font, style, &width, &obj->ascent, &obj->descent);
+		html_painter_calc_text_size_bytes (painter, " ", 1, NULL, NULL, 0, NULL, font, style, &width, &obj->ascent, &obj->descent);
 	}
 
 	HTML_OBJECT (text)->change &= ~HTML_CHANGE_WORD_WIDTH;
@@ -794,14 +810,77 @@ get_next_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 		return html_text_get_nb_width (HTML_TEXT (obj), painter, begin);
 }
 
+static gint
+min_word_width_calc_tabs (HTMLText *text, HTMLPainter *p, gint idx, gint *len)
+{
+	gchar *str, *end;
+	gint rv = 0, line_offset, wt, wl, i;
+	gint epos;
+	gboolean tab = FALSE;
+	
+	if (!html_clueflow_tabs (HTML_CLUEFLOW (HTML_OBJECT (text)->parent), p))
+		return 0;
+
+	/* printf ("tabs %d\n", idx); */
+
+	str = text->text;
+	i = idx;
+	while (i > 0 && *str) {
+		if (*str == ' ')
+			i--;
+
+		str = g_utf8_next_char (str);
+	}
+
+	if (!*str)
+		return 0;
+
+	epos = 0;
+	end = str;
+	while (*end && *end != ' ') {
+		tab |= *end == '\t';
+
+		end = g_utf8_next_char (end);
+		epos++;
+	}
+	
+
+	if (tab) {
+		line_offset = 0;
+		
+		if (idx == 0) {
+			HTMLObject *prev;
+			
+			prev = html_object_prev_not_slave (HTML_OBJECT (text));
+			if (prev && html_object_is_text (prev) && HTML_TEXT (prev)->words > 0) {
+				min_word_width_calc_tabs (HTML_TEXT (prev), p, HTML_TEXT (prev)->words - 1, &line_offset);
+				/* printf ("lo: %d\n", line_offset); */
+			}
+		}
+
+		wl = html_text_text_line_length (str, &line_offset, epos, &wt);
+	} else {
+		wl = epos;
+	}
+	
+	rv = wl - epos;
+		
+	if (len)
+		*len = wl;
+
+	/* printf ("tabs delta %d\n", rv); */
+	return rv;
+}
+
 static guint
-word_width (HTMLText *text, HTMLPainter *p, guint i)
+min_word_width (HTMLText *text, HTMLPainter *p, guint i)
 {
 	g_assert (i < text->words);
 
 	return text->word_width [i]
 		- (i > 0 ? text->word_width [i - 1]
-		   + html_painter_get_space_width (p, html_text_get_font_style (text), text->face) : 0);
+		   + html_painter_get_space_width (p, html_text_get_font_style (text), text->face) : 0)
+		+ min_word_width_calc_tabs (text, p, i, NULL)*html_painter_get_space_width (p, html_text_get_font_style (text), text->face);
 }
 
 /* return non-breakable text width on begin/end of this text */
@@ -819,7 +898,7 @@ html_text_get_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 
 	html_text_request_word_width (text, painter);
 
-	return word_width (text, painter, begin ? 0 : text->words - 1)
+	return min_word_width (text, painter, begin ? 0 : text->words - 1)
 		+ (text->words == 1 ? get_next_nb_width (text, painter, begin) : 0);
 }
 
@@ -831,7 +910,7 @@ get_glyphs (HTMLText *text, HTMLPainter *painter)
 	if (items) {
 		PangoGlyphString *str = NULL;
 		PangoItem *item;
-		GList *il, *gl;
+		GList *il;
 		gchar *heap = NULL, *translated;
 		gint bytes;
 
@@ -843,9 +922,11 @@ get_glyphs (HTMLText *text, HTMLPainter *painter)
 		html_replace_tabs (text->text, translated, bytes);
 		for (il = items; il; il = il->next) {
 			item = (PangoItem *) il->data;
-			str = pango_glyph_string_new ();
-			pango_shape (translated + item->offset, item->length, &item->analysis, str);
-			glyphs = g_list_prepend (glyphs, str);
+			if (item->length) {
+				str = pango_glyph_string_new ();
+				pango_shape (translated + item->offset, item->length, &item->analysis, str);
+				glyphs = g_list_prepend (glyphs, str);
+			}
 		}
 		glyphs = g_list_reverse (glyphs);
 		g_free (heap);
@@ -886,7 +967,7 @@ static gint
 calc_min_width (HTMLObject *self, HTMLPainter *painter)
 {
 	HTMLText *text = HTML_TEXT (self);
-	HTMLObject *obj;
+	HTMLObject *prev, *next, *obj;
 	guint i, w, mw;
 
 	items_destroy (text);
@@ -895,16 +976,28 @@ calc_min_width (HTMLObject *self, HTMLPainter *painter)
 	mw = 0;
 
 	for (i = 0; i < text->words; i++) {
-		w = word_width (text, painter, i);
+		w = min_word_width (text, painter, i);
+		prev = next = NULL;
 		if (i == 0) {
 			obj = html_object_prev_not_slave (self);
-			if (obj && html_object_is_text (obj))
+			if (obj && html_object_is_text (obj)) {
 				w += html_text_get_nb_width (HTML_TEXT (obj), painter, FALSE);
-		} else if (i == text->words - 1) {
-			obj = html_object_next_not_slave (self);
-			if (obj && html_object_is_text (obj))
-				w += html_text_get_nb_width (HTML_TEXT (obj), painter, TRUE);
+				prev = obj;
+			}
 		}
+		if (i == text->words - 1) {
+			obj = html_object_next_not_slave (self);
+			if (obj && html_object_is_text (obj)) {
+				w += html_text_get_nb_width (HTML_TEXT (obj), painter, TRUE);
+				next = obj;
+			}
+		}
+
+		if (prev && prev->min_width < w)
+			prev->min_width = w;
+		if (next && next->min_width < w)
+			next->min_width = w;
+
 		if (w > mw)
 			mw = w;
 	}
@@ -1389,7 +1482,7 @@ append_selection_string (HTMLObject *self,
 	last = html_text_get_text (text,
 				   text->select_start + text->select_length);
 	*/
-	g_string_append_len (buffer, p, last - p);
+	html_engine_save_string_append_nonbsp (buffer, p, last - p);
 
 }
 
@@ -1455,18 +1548,17 @@ get_cursor_base (HTMLObject *self,
 				HTMLText *text;
 				GtkHTMLFontStyle font_style;
 				gint line_offset, width, asc, dsc;
+				gchar *slave_text;
 
 				text = HTML_TEXT (self);
 
 				font_style = html_text_get_font_style (text);
-				line_offset = html_text_slave_get_line_offset (slave,
-									       html_text_get_line_offset (HTML_TEXT (self),
-													  painter),
-									       slave->posStart, painter);
+				line_offset = html_text_slave_get_line_offset (slave, 0, painter);
 				/* FIXME: cache items and glyphs? */
+				slave_text = html_text_get_text (text, slave->posStart);
 				html_painter_calc_text_size (painter,
-							     html_text_get_text (text, slave->posStart),
-							     offset - slave->posStart, NULL, NULL, &line_offset,
+							     slave_text,
+							     offset - slave->posStart, NULL, NULL, 0, &line_offset,
 							     font_style, text->face, &width, &asc, &dsc);
 
 				*x += width;
@@ -1868,7 +1960,7 @@ html_text_magic_link (HTMLText *text, HTMLEngine *engine, guint offset)
 	/* check forward to ensure chars are < 0x80, could be removed once we have utf8 regex */
 	do {
 		cur = g_utf8_next_char (cur);
-		if (cur && *cur && g_utf8_get_char (cur) >= 0x80)
+		if (cur && *cur && (uc = g_utf8_get_char (cur)) >= 0x80)
 			exec = FALSE;
 	} while (exec && cur && *cur && uc != ' ' && uc != ENTITY_NBSP);
 
@@ -1889,18 +1981,14 @@ html_text_magic_link (HTMLText *text, HTMLEngine *engine, guint offset)
 	}
 
 	if (exec) {
-		while (offset < text->text_len && !rv) {
-			for (i=0; i<MIM_N; i++) {
-				if (mim [i].preg && !regexec (mim [i].preg, str, 2, pmatch, 0)) {
-					paste_link (engine, text,
-						    g_utf8_pointer_to_offset (text->text, str + pmatch [0].rm_so),
-						    g_utf8_pointer_to_offset (text->text, str + pmatch [0].rm_eo), mim [i].prefix);
+		for (i=0; i<MIM_N; i++) {
+			if (mim [i].preg && !regexec (mim [i].preg, str, 2, pmatch, 0)) {
+				paste_link (engine, text,
+					    g_utf8_pointer_to_offset (text->text, str + pmatch [0].rm_so),
+					    g_utf8_pointer_to_offset (text->text, str + pmatch [0].rm_eo), mim [i].prefix);
 					rv = TRUE;
 					break;
-				}
 			}
-			str = g_utf8_next_char (str);
-			offset++;
 		}
 	}
 
